# ipencoder6.tcl---IP-Encoder-6 IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ipencoder6
set frametitle "IP-Encoder-6 Diagnostics Frame"
set icontitle "ipencoder6"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource vme}

set mname "IPENCODER6"

set VIPC616BaseAddress 0xffffc000
set IPslot 3

loadsource ipcommon


proc define-module {} {
  global resource mname IPslot VIPC616BaseAddress
  set-footer "deleting all IP-Encoder-6 registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

#     Counters
     define-register xIP Ch1.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x00 -d io -p ro -w 16 -zx"
     define-register xIP Ch2.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x02 -d io -p ro -w 16 -zx"
     define-register xIP Ch3.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x04 -d io -p ro -w 16 -zx"
     define-register xIP Ch4.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x06 -d io -p ro -w 16 -zx"
     define-register xIP Ch5.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x08 -d io -p ro -w 16 -zx"
     define-register xIP Ch6.Counter   "-a $VIPC616BaseAddress -s $IPslot -o 0x0a -d io -p ro -w 16 -zx"

     define-register xIP Reset         "-a $VIPC616BaseAddress -s $IPslot -o 0x0c -d io -p ro -w 16 -zx"
     define-register xIP Overflow      "-a $VIPC616BaseAddress -s $IPslot -o 0x10 -d io -p rw -w 16 -zx"
     define-register xIP Index         "-a $VIPC616BaseAddress -s $IPslot -o 0x12 -d io -p rw -w 16 -zx"
     define-register xIP UpDown        "-a $VIPC616BaseAddress -s $IPslot -o 0x14 -d io -p ro -w 16 -zx"

#    Update IP details

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0xaa} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

     click-redisplay
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-redisplay {} {

    for {set i 1} {$i <= 6} {incr i} {
        update-widget CH$i [read-register Ch$i.Counter]
    }
    update-widget OVFLOW [read-register Overflow]
    update-widget INDEX  [read-register Index]
    update-widget UPDOWN [read-register UpDown]
}

proc click-reset {args} {read-register Reset; click-redisplay}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }

      CLEAROVFLOW  {write-register Overflow 0; click-redisplay}
      CLEARINDEX   {write-register Index 0; click-redisplay}

      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env VIPC616BaseAddress IPslot

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 12
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D]         :value $IPslot
      :next-row T
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15

      :message M1 :label "Counters" :bold T
       :next-row T
       for {set i 1} {$i <= 6} {incr i} {
           :text CH$i :label $i  :w 6
       }
       :next-row T
      :message M2 :label "Registers" :bold T
       :next-row T
        :text OVFLOW :label Overflow  :w 6
        :text INDEX  :label Index     :w 6
        :text UPDOWN :label Up/Down   :w 6
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15; :row-gap 10
      :button RESET        :label "Reset"              :command click-reset
      :button CLEAROVFLOW  :label "Clear Overflow" 
      :button CLEARINDEX   :label "Clear Index" 
       :next-row T
      :button REDISPLAY :label "Redisplay"              :command click-redisplay
      :button DEFINE    :label "Define IP-Encoder-6 registers" :command define-module
   
   :show T
}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}




#proc read-register  {r}   {global $r; return [set $r]}
#proc write-register {r v} {global $r; set $r $v}
