# V767.tcl---CAEN V767 TDC module control frame



set frame "V767"
set frametitle "CAEN V767 TDC Setup & Monitor"
set icontitle "V767"

set modulename v767
set moduletype  "CAEN V767"

set defaultmodulelist [list 1 2 3 4 5 6 7 8]

set modulelist {???}
set moduleindex 0

set maxmodule 8
set maxtdc 128

set maxmodule [inform vmecontrol inquire [set modulename]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [lindex $modulelist $moduleindex]
set tdc  0

set alltdcs 0
set allmodules 0

set adjustment 0
set adjset 0
set adjchan 1

set w2r(GEO)        "GEO_Address"
set w2r(GEOADDRESS) "GEO_Address"
set w2r(INTRLEV)    "Intr_Level"
set w2r(INTRVEC)    "Intr_Vec"
set w2r(MCST)       "MCST/CBLT_Ctrl"
set w2r(DRMODE)     "Ctrl2.DrMode"

set w2r(CLEAR)      "Clear"
set w2r(EVCLR)      "EvCounterClr"

set r2w(ACQMODE)    "Ctrl2.AcqMode"
set r2w(GEO)        "GEO_Address"
set r2w(EVCOUNTER)  "EvCounter"
set r2w(AUTOLOAD)   "read_auto_load"

set electronicsvisible 0
set electronicsframe "[self frame-name].electronics"

set savedirectory CAENtdcParameters

loadsource common.vme

set expertlist {                                                    \
   {"Clear Event Counter"               "click-widget EVCLR 0"}           \
   {"Load Default V767 Configuration"   "do_cmd_opcode 0x1500"}  \
   {"Save User V767 Configuration"      "do_cmd_opcode 0x1600"}  \
   {"Load User V767 Configuration"      "do_cmd_opcode 0x1700"}  \
   {"Enable Auto Load"                  "do_cmd_opcode 0x1800"}  \
   {"Disable Auto Load"                 "do_cmd_opcode ox1900"}  \
}



proc Save_Parameters {} {
   global env savedirectory
   global modulename module

   catch {file delete [file join $env(EXPERIMENTBASE) $savedirectory]}
   
   if {![file exists [file join $env(EXPERIMENTBASE) $savedirectory]]} then {
      set z [catch {:file mkdir [file join $env(EXPERIMENTBASE) $savedirectory]} m]
      if {$z != 0} then {midas-warning "Cannot create directory [file join $env(EXPERIMENTBASE) $savedirectory]\n $m"; return}
   }

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set modulelist {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

   set module_SAVE $module   ;# currently selected

   set-busy
   set contents ""
   set r 0
   foreach module $modulelist {
         set-footer "Saving settings for module $modulename#$module..."

         click-select-module $module

         set z [catch {open [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module] w} f]
         if {$z != 0} then {
             midas-warning "Cannot create file [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module]\n $f"
             set r 1
             break
         }

#    obtain values for this module

         set vl [do_read_opcode 0x1400]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "AcqMde14 $vl"

         set vl [do_read_opcode 0x2600]  ;# reads 8 words
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Enble26  $vl"

         set vl [do_read_opcode 0x3100]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Trig31 $vl"

         set vl [do_read_opcode 0x3300]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Trig33 $vl"

         set vl [do_read_opcode 0x3500]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Trig35 $vl"

         set vl [do_read_opcode 0x3a00]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Trig3a $vl"

         set vl [do_read_opcode 0x4700]  ;# reads 2 words
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Strt47  $vl"

         set vl [do_read_opcode 0x6700]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "Edge67  $vl"

         set vl [do_read_opcode 0x7300]
         if {$vl == 0x5aa5} then {
             midas-warning "Error reading module parameters\nIf this continues you may need to Reset the module"
             set r 1
             break
         }
         puts $f "DRdy73  $vl"

         close $f

         append contents "Settings saved for module $modulename#$module to file [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module]\n"
   }
   clear-busy

   catch {close $f}

   if {$r != 0} then {
      midas-warning "Save of module settings to $env(EXPERIMENTDIR)/$savedirectory abandoned"
      clear-footer
      return
   }

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been saved!"
   } else {
      set-footer "Module settings saved to [file join $env(EXPERIMENTBASE) $savedirectory]"
      midas-report "$contents"
   }

   click-select-module $module_SAVE
}

set Verbose 1
proc Restore_Parameters {} {global Verbose; set Verbose 0; do_Restore_Parameters; set Verbose 1}

proc do_Restore_Parameters {} {
   global env Verbose savedirectory
   global module modulename

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set modulelist {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

   set module_SAVE $module   ;# currently selected

   set-busy
   set contents ""
   set r 0
   foreach module $modulelist {
         set-footer "Restoring settings for module $modulename#$module..."

         click-select-module $module

         set z [catch {open [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module] r} f]
         if {$z != 0} then {
             if {$Verbose} then {
                midas-warning "Cannot access file [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module]\n $f"
             } else {
                insert-log "**Warning -  Cannot access file [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module]\n $f"
             }
             set r 1
             break
         }

#    read module settings from file
         set Settings [read $f]
         close $f

#AcqMde14 0
#Trig31 300 
#Trig33 -250 
#Trig35 0 
#Trig3a 3 
#Edge67  1 1 1 
#DRdy73  2 
#Enble26  255 255 255 255 255 255 255 255 
#Strt47  9 

         foreach item [split $Settings \n] {
             if {$item == ""} {continue}
             set p [first $item]
             set vl [tail $item]

             switch $p {
                 AcqMde14  {
                     if {[expr $vl & 3] == 0} then {do_cmd_opcode 0x1000}
                     if {[expr $vl & 3] == 1} then {do_cmd_opcode 0x1100}
                     if {[expr $vl & 3] == 2} then {do_cmd_opcode 0x1200}
                     if {[expr $vl & 3] == 3} then {do_cmd_opcode 0x1300}
                 }

                 Trig31 {
                     do_set_opcode  0x3000 $vl
                 }

                 Trig33 {
                     do_set_opcode  0x3200 $vl
                 }

                 Trig35 {
                     do_set_opcode  0x3400 $vl
                 }

                 Trig3a {
                     if {[expr $vl & 1] == 0} then {do_cmd_opcode 0x3700} else {do_cmd_opcode 0x3600}
                     if {[expr $vl & 2] == 0} then {do_cmd_opcode 0x3900} else {do_cmd_opcode 0x3800}
                 }

                 Edge67 {          ;#  reads 3 words
                     if {[expr [first $vl] & 3] == 1} then {do_cmd_opcode 0x6000}
                     if {[expr [first $vl] & 3] == 2} then {do_cmd_opcode 0x6100}
                     if {[expr [first $vl] & 3] == 3} then {do_cmd_opcode 0x6600}
                     if {[expr [second $vl] & 3] == 1} then {do_cmd_opcode 0x6000}
                     if {[expr [second $vl] & 3] == 2} then {do_cmd_opcode 0x6100}
                     if {[expr [second $vl] & 3] == 3} then {do_cmd_opcode 0x6600}
                     if {[expr [third $vl] & 3] == 1} then {do_cmd_opcode 0x6400}
                     if {[expr [third $vl] & 3] == 2} then {do_cmd_opcode 0x6500}
                     if {[expr [third $vl] & 3] == 3} then {do_cmd_opcode 0x6600}
                 }

                 DRdy73 {
                     if {[expr $vl & 3] == 0} then {do_cmd_opcode 0x7000}
                     if {[expr $vl & 3] == 1} then {do_cmd_opcode 0x7100}
                     if {[expr $vl & 3] == 2} then {do_cmd_opcode 0x7200}
                 }

                 Enble26 {
                     do_set_opcode 0x4400 $vl   ;# write 8 values
                 }

                 Strt47 {
                     if {[expr $vl & 3] == 0} then {do_cmd_opcode 0x4200}
                     if {[expr $vl & 3] == 1} then {do_cmd_opcode 0x4000}
                     if {[expr $vl & 3] == 2} then {do_cmd_opcode 0x4100}
                     if {[expr $vl & 4] == 4} then {do_cmd_opcode 0x4300}
                     if {[expr $vl & 8] == 8} then {do_cmd_opcode 0x4500}
                 }
              }
         }
         append contents "Settings restored for module $modulename#$module from file [file join $env(EXPERIMENTBASE) $savedirectory $modulename#$module]\n"
         
   }

   clear-busy

   if {$r != 0} then {
      if {$Verbose} then {
          midas-warning "Restore of module settings from $env(EXPERIMENTDIR)/$savedirectory abandoned"
      } else {
          insert-log "**Warning  -  Restore of module settings from $env(EXPERIMENTDIR)/$savedirectory abandoned"
      }
      clear-footer
      return
   }

   if {$contents == ""} then {
      if {$Verbose} then {midas-warning "Warning - Nothing has been restored!"}
   } else {
      if {$Verbose} then {
          set-footer "Module settings restored from [file join $env(EXPERIMENTBASE) $savedirectory]"
          midas-report "$contents"
      } else {
          insert-log "$contents"
      }
   }

   click-select-module $module_SAVE

}


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}


proc click-widget {w v} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)
    set value $v

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc update-widget {w} {
    global r2w
    global modulename module

    set register $r2w($w)
    :modify $w :value [read-vme-register $modulename#$module.$register]
}

proc do_read_register {r} {
    global modulename module
    return [read-vme-register $modulename#$module.$r]
}

proc do_cmd_opcode {OpCode} {
    global modulename module modulelist
    global allmodules
    global EGreport
    
    set EGreport_SAVE $EGreport
    set EGreport -1
    set-busy

    if {$allmodules == 0} then {
        set modules $module
    } else {
        set modules $modulelist
    }
    
    foreach m $modules {
    
      for {set i 50} {$i > 0} {incr i -1} {
        set vl [read-vme-register $modulename#$m.OpCodeCtrl]
        if {$vl == 0x0002} then {
            after 15
            write-vme-register $modulename#$m.OpCode $OpCode
            break
        }
        after 20
      }
      if {$i == 0} then {midas-warning "do_cmd_opcode: timeout for write access: $modulename#$m.OpCodeCtrl = $vl"}
    }
    
    clear-busy
    set EGreport $EGreport_SAVE
}

proc do_set_opcode {OpCode value} {
    global modulename module modulelist
    global allmodules
    global EGreport
    
    set EGreport_SAVE $EGreport
    set EGreport -1
    set-busy

    if {$allmodules == 0} then {
        set modules $module
    } else {
        set modules $modulelist
    }
    
    foreach m $modules {
    
      for {set i 50} {$i > 0} {incr i -1} {
        set vl [read-vme-register $modulename#$m.OpCodeCtrl]
        if {$vl == 0x0002} then {
            after 15
            write-vme-register $modulename#$m.OpCode $OpCode
            break
        }
        after 20
      }
      if {$i == 0} then {midas-warning "do_set_opcode: timeout for write access: $modulename#$m.OpCodeCtrl = $vl"}
      
      after 20
      
      set nW 1
      if {$OpCode == 0x2500} {set nW 8}
      
      for {set i 50} {$i > 0} {incr i -1} {
        set vl [read-vme-register $modulename#$m.OpCodeCtrl]
        if {$vl == 0x0002} then {
            after 15

            for {set k 0} {$k < $nW} {incr k} {
                write-vme-register $modulename#$m.OpCode [lindex $value $k]
            }
            
            break
        }
        after 20
      }
      if {$i == 0} then {midas-warning "do_set_opcode: timeout for write access: $modulename#$m.OpCodeCtrl = $vl"}
      
    }
    
    clear-busy
    set EGreport $EGreport_SAVE
}

proc do_read_opcode {OpCode} {
    global modulename module
    global EGreport
    
    set EGreport_SAVE $EGreport
    set EGreport -1
    set-busy

    set m $module
     
   for {set j 3} {$j > 0} {incr j -1} {
    
      for {set i 50} {$i > 0} {incr i -1} {
        set vl [read-vme-register $modulename#$m.OpCodeCtrl]
        if {$vl == 0x0002} then {
            after 15
            write-vme-register $modulename#$m.OpCode $OpCode
            break
        }
        after 20
      }
      if {$i == 0} then {
      
          clear-busy
          set EGreport $EGreport_SAVE
    
          midas-warning "do_read_opcode: timeout for write access: $modulename#$m.OpCodeCtrl = $vl"
          return 0x5aa5
      }
      
      after 20
      
      set nR 1
      if {$OpCode == 0x2600} {set nR 8}
      if {$OpCode == 0x6700} {set nR 3}

      for {set i 50} {$i > 0} {incr i -1} {
        set vl [read-vme-register $modulename#$m.OpCodeCtrl]
        if {$vl == 0x0001} then {
            after 15
            
            set vl ""
            for {set k 0} {$k < $nR} {incr k} {
                append vl [format "%hd " [read-vme-register $modulename#$m.OpCode]]
            }

            clear-busy
            set EGreport $EGreport_SAVE
            return $vl
        }
        after 20
      }
      if {$i == 0} then {set-footer "do_read_opcode: timeout for read access: $modulename#$m.OpCodeCtrl = $vl"}
   }
   
    clear-busy
    set EGreport $EGreport_SAVE
    
   set-footer "do_read_opcode: timeout for read access after 3 attempts: $modulename#$m.OpCodeCtrl = $vl"
   
   return 0x5aa5
}

proc click_acqmode {arg} {

    switch [first $arg] {

       0   {do_cmd_opcode 0x1000}
       1   {do_cmd_opcode 0x1100}
       2   {do_cmd_opcode 0x1200}
       3   {do_cmd_opcode 0x1300}
    }

    after 200
    update-widget ACQMODE
}

proc read_acqmode {} {
    global r2w
    global modulename module

    set register $r2w(ACQMODE)
    return [read-vme-register $modulename#$module.$register]
}


#    3x opcodes   -   Trigger setup
#    30   set window width
#    32   set window offset
#    34   set trigger latency
#    36   enable subtraction of trigger time
#    37   disable subtraction of trigger time
#    38   enable overlapping triggers
#    39   disable overlapping triggers

proc do_3x_opcode {w v} {

    switch $w {
       OPT30  {do_set_opcode 0x3000 $v}
       OPT32  {do_set_opcode 0x3200 $v}
       OPT34  {do_set_opcode 0x3400 $v}
       OPT36  {do_opt36 $v}
       OPT38  {do_opt38 $v}
    }
    redisplay_opt3x
}

proc do_opt36 {v} {

    switch $v {
      0   {do_cmd_opcode 0x3600}
      1   {do_cmd_opcode 0x3700}
    }
}

proc do_opt38 {v} {

    switch $v {
      0   {do_cmd_opcode 0x3800}
      1   {do_cmd_opcode 0x3900}
    }
}

proc  redisplay_opt3x {} {
    
    :modify OPT30 :value [do_read_opcode 0x3100]
    :modify OPT32 :value [do_read_opcode 0x3300]
    :modify OPT34 :value [do_read_opcode 0x3500]

    set vl [do_read_opcode 0x3a00]
    :modify OPT36  :value [expr $vl & 1]
    :modify OPT38  :value [expr ($vl >> 1) & 1]
}

proc do_6x_opcode {w} {

    switch $w {
       OPT60  {do_cmd_opcode 0x6000}
       OPT61  {do_cmd_opcode 0x6100}
       OPT62  {do_cmd_opcode 0x6200}
       OPT63  {do_cmd_opcode 0x6300}
       OPT64  {do_cmd_opcode 0x6400}
       OPT65  {do_cmd_opcode 0x6500}
       OPT66  {do_cmd_opcode 0x6600}
    }
    redisplay_opt6x
}

proc redisplay_opt6x {} {

    :modify OPT60 :value 0
    :modify OPT61 :value 0
    :modify OPT62 :value 0
    :modify OPT63 :value 0
    :modify OPT64 :value 0
    :modify OPT65 :value 0
    :modify OPT66 :value 0

    set vl [do_read_opcode 0x6700]

    switch [first $vl] {
        1 {
           switch [second $vl] {
               1  {:modify OPT60 :value 1}
               2  {:modify OPT63 :value 1}
           }
        }
        2 {
           switch [second $vl] {
               1  {:modify OPT62 :value 1}
               2  {:modify OPT61 :value 1}
           }
        }
    }

    switch [third $vl] {
        1 {:modify OPT64 :value 1}
        2 {:modify OPT65 :value 1}
        3 {:modify OPT66 :value 1}
    }
}

proc do_4x_opcode {w} {

    switch $w {
       OPT40  {do_cmd_opcode 0x4000}
       OPT41  {do_cmd_opcode 0x4100}
       OPT42  {do_cmd_opcode 0x4200}
       OPT43  {do_cmd_opcode 0x4300}
       OPT44  {do_cmd_opcode 0x4400}
       OPT45  {do_cmd_opcode 0x4500}
       OPT46  {do_cmd_opcode 0x4600}
    }
    redisplay_opt4x
}

proc redisplay_opt4x {} {

    set vl [do_read_opcode 0x4700]

    switch [expr $vl & 3] {
        0 {:modify OPT42 :value 1; :modify OPT40 :value 0; :modify OPT41 :value 0}
        1 {:modify OPT40 :value 1; :modify OPT41 :value 0; :modify OPT42 :value 0}
        2 {:modify OPT41 :value 1; :modify OPT40 :value 0; :modify OPT42 :value 0}
    }

    switch [expr ($vl >> 2) & 1] {
        0 {:modify OPT43 :value 0; :modify OPT44 :value 1}
        1 {:modify OPT43 :value 1; :modify OPT44 :value 0}
    }

    switch [expr ($vl >> 3) & 1] {
        0 {:modify OPT45 :value 0; :modify OPT46 :value 1}
        1 {:modify OPT45 :value 1; :modify OPT46 :value 0}
    }
}

proc do_7x_opcode {w v} {

    switch $w {
       OPT70  {do_cmd_opcode 0x7000}
       OPT71  {do_cmd_opcode 0x7100}
       OPT72  {do_cmd_opcode 0x7200}
       OPT74  {do_set_opcode 0x7400 $v}
    }
    redisplay_opt7x
}

proc redisplay_opt7x {} {

    :modify OPT74 :value [do_read_opcode 0x7500]

    :modify OPT70 :value 0
    :modify OPT71 :value 0
    :modify OPT72 :value 0

    set vl [do_read_opcode 0x7300]

    switch [expr $vl] {
        0   {:modify OPT70 :value 1}
        1   {:modify OPT71 :value 1}
        2   {:modify OPT72 :value 1}
    }
}

proc click-enable {v} {

    switch $v {
       0  {select_ch_enable; do_cmd_opcode 0x2300}
       1  {select_ch_enable; do_cmd_opcode 0x2400}
       2  {select_ch_enable}
    }
    :modify ENABLE :value -1

    redisplay_opt2x
}

proc click-ch-select {i} {

    if {[[:path PATT].bit$i get] == 0} then {
        do_cmd_opcode [expr 0x2000 + $i]
    } else {
        do_cmd_opcode [expr 0x2100 + $i]
    }

    set vl [do_read_opcode [expr 0x2200 + $i]]

    [:path PATT].bit$i delete 0 end
    [:path PATT].bit$i insert 0 $vl
}

proc redisplay_opt2x {} {

     set vl [do_read_opcode 0x2600]

     set i 0
     for {set ii 0} {$ii < 8} {incr ii} {
        set p [lindex $vl $ii]
        for {set jj 0} {$jj < 16} {incr jj} {
           set e [expr ($p >> $jj) & 1]
           [:path PATT].bit$i delete 0 end
           [:path PATT].bit$i insert 0 "$e"
           incr i
        }
     }
}

proc do_50_opcode {v} {
    global adjustment adjset

    set adjustment $v

    for {set i 0} {$i < 128} {incr i} {
       if {[[:path PATT].bit$i get] == 1 || $adjset == 0} then {
          set-footer  "setting ch $i adjust to $adjustment"
          do_set_opcode [expr 0x5000 + $i] $adjustment
          after 1000
          clear-footer
       }
    }
}

proc do_51_opcode {i} {
    global adjchan
    
    set adjchan $i
    :modify ADJVAL :value [do_read_opcode [expr 0x5100 + $i]]
}

proc click-opt5x-select {i} {

    if {[[:path PATT].bit$i get] == 0} then {
        set vl 1
    } else {
        set vl 0
    }
    [:path PATT].bit$i delete 0 end
    [:path PATT].bit$i insert 0 $vl
}

proc do_channel_adj_enable {v} {

   if {$v == 1} then {do_cmd_opcode 0x5400} else {do_cmd_opcode 0x5500}
   :modify ADJENB :value [do_read_opcode 0x5600]
}

proc redisplay_opt5x {} {

   :modify OFFSET :value [do_read_opcode 0x5300]
}

proc reset-module {} {
    global modulename module allmodules

    set register Bit_Set
    set data 0x80

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $data
    } else {
        write-vme-registers $modulename#\[?,??\].$register $data
    }

    set register Bit_Clear
    set data 0x80

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $data
    } else {
        write-vme-registers $modulename#\[?,??\].$register $data
    }
    
    set-footer "Waiting 2secs for module to reset"
    after 2000
    clear-footer
    
    click-redisplay
}

proc flush-module {} {

#    data fifo flush - module clear does not work!!
       for {set Offset 0} {$Offset < 2048} {incr Offset 1} {
          set word [do_read_register Output_Word]
          if {[expr $word >> 21 & 3] == 3} {break}    ;#  end of data 
       }
}

proc do_read_buffer {} {
  global CurrentVMEServer modulename module
  global EGreport
  
  set EGreport_Save $EGreport
  set EGreport -1
  set-busy

  set info ""
  for {set Offset 0} {$Offset < 2048} {incr Offset 1} {

     set register $modulename#$module.Output_Word
     set data [string trimleft [read-vme-register $register]]

     append info "\nOutput Buffer (FIFO) Offset $Offset\n\n"
     foreach word $data {
         set type [expr $word >> 21 & 3]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 24 & 0x7f]
             set st [expr $word >> 23 & 1]
             set ed [expr $word >> 20 & 1]
             set tdc [expr $word  & 0xfffff]
             append info "[format "0x%8x    %8s   channel= %5i; start= %3i; edge= %3i; time= 0x%5x" $word $event_type $channel $st $ed $tdc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0x3ff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %6i" $word $event_type $geo $counter]"
            }
            1 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set status [expr $word >> 24 & 0x1f]
             set counter [expr $word & 0xffff]
             append info "[format "0x%8x    %8s   module= %5i; status=%2i; event data counter= %8i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - assumed end of data\n"
             midas-information $info
             clear-busy
             set EGreport $EGreport_Save
             return
            }
         }
         append info "\n"
     }
  }
  append info "end of output buffer reached\n"
  midas-information $info

  clear-busy
  set EGreport $EGreport_Save
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
   if {$electronicsvisible==0} then {make-electronics} else {open-frame $electronicsframe}
   click-redisplay
}

proc click-registers {} {
   global EGreport moduletype modulename module registers_table

   if {[info procs [set moduletype]_registers] == ""} then  \
        {set-footer "No [set moduletype]_registers procedure"; return}

   [set moduletype]_registers

   clear-footer
   set EGreport -1

   set info "[set moduletype] - module $modulename#$module\n\n"

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $modulename#$module.$register]
       append info "[format "%-40s %-20s" $description $data]\n"
   }

  midas-information $info

   set EGreport 0
}

proc click-select-module {v} {
   global debug module maxmodule
   if {$debug} then {puts stderr "click-select-module with argument \{$v\}"}
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-select-tdc {v} {}

proc click-all-tdcs {v} {global alltdcs; set alltdcs $v}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc repaint {} {}

proc redisplay-module {} {
   global w2r modulename module maxmodule allmodules

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module
   :modify ALL-MODULES :value $allmodules

   foreach w {GEO ACQMODE EVCOUNTER} {update-widget $w}
}

proc redisplay-tdcs {} {
    global w2r
    global modulename module maxtdc
}

proc redisplay-electronics {} {
   global electronicsvisible
   global modulename module w2r

   if {$electronicsvisible==0} then {return}

   foreach w {GEOADDRESS INTRLEV INTRVEC MCST DRMODE} {
       set register $w2r($w)
       :modify $w :value [read-vme-register $modulename#$module.$register]
   }

   :modify AUTOLOAD :value [do_read_opcode 0x1a00]

#    Status register 1

   set register "Status1"
   set value [read-vme-register $modulename#$module.$register]

   :modify DREADY     :value [expr $value & 1]
   :modify BUSY       :value [expr $value >> 2 & 1]
   :modify GDREADY    :value [expr $value >> 1 & 1]
   :modify GBUSY      :value [expr $value >> 3 & 1]
   :modify TERM       :value [expr $value >> 6 & 3]

#    Status register 2

   set register "Status2"
   set value [read-vme-register $modulename#$module.$register]

   :modify BEMPTY     :value [expr $value & 1]
   :modify BFULL      :value [expr $value >> 1 & 1]
   :modify BPART      :value [expr $value >> 2 & 1]
   :modify TDCERR     :value [expr $value >> 3 & 1]

}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module
   redisplay-tdcs
   redisplay-electronics

   clear-busy
   set EGreport 0
}

proc start args {
   global started modulename
   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   loadsource [set modulename].defn

   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}


proc click arg {
   global debug expertlist adjset
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      ALL-TDCS    {click-all-tdcs $v}

      CLEAR      {click-widget $w 0}

      OPT3*      {do_3x_opcode $w $v}
      OPT4*      {do_4x_opcode $w}
      OPT6*      {do_6x_opcode $w}
      OPT7*      {do_7x_opcode $w $v}

      OFFSET     {do_set_opcode 0x5200 $v; redisplay_opt5x}

      ADJUSTMENT {do_50_opcode $v}
      ADJSET     {set adjset $v}
      ADJENB     {do_channel_adj_enable $v}
      ADJVAL     {do_51_opcode $v}

      ENABLE     {click-enable $v}

      SAVE       {do_cmd_opcode 0x1600; do_cmd_opcode 0x1800}
      LOAD       {do_cmd_opcode 0x1700}
      DEFAULT    {do_cmd_opcode 0x1500}


      ENGINEERING       {click-$v}
      REDISPLAY  	{click-redisplay}

      EXPERT  {
         set action [lindex $expertlist $i]
         set r [midas-confirmation "You are about to execute the action \"[first $action]\"\n\nAre you sure?"]
         if {$r == 1} then {eval [second $action]}
      }

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p [list $v $i]} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env expertlist
  global module maxmodule modulelist maxtdc

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist

     :col-gap 10
     :text GEO :label "GEO Address" :w 6 :deaf T
     :text EVCOUNTER :label "Event Counter" :w 8 :deaf T
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T
     :choice ONLINE    :label "TDC controller"  :strings [list Online Offline]
     :choice ENABLE    :label "Channel Enable"  :strings [list "Enable All" "Disable All" "Select"]
     :slider OFFSET :label "Global Offset" :min 0 :max 65535
  
     :next-row T
     :choice ACQMODE :label "Acquisition Mode" :w 22 :strings \
          [list "Stop Trigger Matching" "Start Trigger Matching" "Start Gating" "Continuous Storage"]

     :next-row T
     :button TRIG   :label "Trigger Options"         :command select_trigger_options
     :button EDGE   :label "Edge Detection Options"  :command select_edge_detection_options
     :button START  :label "Start Options"           :command select_start_options
     :button DATA   :label "Data Ready Options"      :command select_dr_options
     :button ADJUST :label "Channel Adjust"          :command select_adj_options

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button SAVE    :label "Save Setup"    :command Save_Parameters
      :button RESTORE :label "Restore Setup" :command do_Restore_Parameters
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software registers]
      :button MEMRD   :label "read output buffer" :command do_read_buffer

      set menu {}
      foreach item $expertlist {lappend menu [first $item]}
      :menubutton EXPERT :label "Expert"  :strings $menu

  :show T

}

proc select_trigger_options {} {

 set valuex 24c

  :frame "[self frame-name].optsel" :label "CAEN V767 Triggering Options" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

     :slider OPT30   :label "Window Width"  :min 1 :max 34000 :valuex $valuex
     :slider OPT32   :label "Window Offset" :min -32000 :max +32768 :valuex $valuex
     :slider OPT34   :label "Latency" :min 0 :max 50 :valuex $valuex
     :choice OPT36   :label "Subtraction of Trigger Time" :strings [list Enable Disable] :valuex $valuex
     :choice OPT38   :label "Overlapping Windows"  :strings [list Enable Disable] :valuex $valuex

     :button UPDATEOPT3x   :label "Update"   :command redisplay_opt3x

  :show T

  redisplay_opt3x
}

proc select_edge_detection_options {} {

 set valuex 48c

  :frame "[self frame-name].optsel" :label "CAEN V767 Edge Detection Options" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

     :checkbox OPT60    :label "Rising edge only on all channels" :valuex $valuex
     :checkbox OPT61    :label "Falling edge only on all channels" :valuex $valuex
     :checkbox OPT62    :label "Rising edge odd channels; Falling edge even channels" :valuex $valuex
     :checkbox OPT63    :label "Falling edge odd channels; Rising edge even channels" :valuex $valuex
     :checkbox OPT64    :label "Rising edge only start" :valuex $valuex
     :checkbox OPT65    :label "Falling edge only start" :valuex $valuex
     :checkbox OPT66    :label "Both Edges on all channels" :valuex $valuex

     :button UPDATEOPT6x   :label "Update"   :command redisplay_opt6x

  :show T

  redisplay_opt6x
}

proc select_start_options {} {

 set valuex 40c

  :frame "[self frame-name].optsel" :label "CAEN V767 Start Options" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

     :checkbox OPT40    :label "Enable readout of start time" :valuex $valuex
     :checkbox OPT41    :label "Enable readout of 4 start times" :valuex $valuex
     :checkbox OPT42    :label "Disable readout of start time" :valuex $valuex
     :checkbox OPT43    :label "Enable subtraction of start time" :valuex $valuex
     :checkbox OPT44    :label "Disable subtraction of start time" :valuex $valuex
     :checkbox OPT45    :label "Enable empty start" :valuex $valuex
     :checkbox OPT46    :label "Disable empty start" :valuex $valuex

     :button UPDATEOPT4x   :label "Update"   :command redisplay_opt4x

  :show T

  redisplay_opt4x
}

proc select_dr_options {} {

 set valuex 32c

  :frame "[self frame-name].optsel" :label "CAEN V767 Data Ready Options" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

     :checkbox OPT70    :label "Set Data Ready = Event Ready" :valuex $valuex
     :checkbox OPT71    :label "Set Data Ready = Buffer Almost Full" :valuex $valuex
     :checkbox OPT72    :label "Set Data Ready = Buffer Not Empty" :valuex $valuex
     :slider   OPT74    :label "Set almost full level"  :min 2 :max 16383 :valuex $valuex

     :button UPDATEOPT6x   :label "Update"   :command redisplay_opt7x

  :show T

  redisplay_opt7x
}

proc select_ch_enable {} {
    global env

  :frame "[self frame-name].optsel" :label "CAEN V767 Channel Enable" :resize F :transient T

   :panel P :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

        :canvas PATT  :w 400 :h 270
        [:path PATT] config -background $env(tkframebackground)

        set x 20
        set y 20
        set i 0
        for {set ii 0} {$ii < 8} {incr ii} {

           :draw text :value $x $y -text "$i"
           incr x 30

           for {set jj 0} {$jj < 16} {incr jj} {
              entry [:path PATT].bit$i -relief flat -width 1 -justify right
              [:path PATT].bit$i insert 0 "0"
              :draw window :tag PATTbit$i :value $x $y -window [:path PATT].bit$i 
              bind [:path PATT].bit$i <ButtonPress-1>  "click-ch-select $i"
              incr x 20
              incr i
           }

           incr x 10

           :draw text :value $x $y -text "[expr $i-1]"

           incr y 32
           set x 20
        }

     :button UPDATEOPT2x   :label "Update"   :command redisplay_opt2x

  :show T
}

proc select_adj_options {} {
    global env adjustment adjset

 set valuex 32c

  :frame "[self frame-name].optsel" :label "CAEN V767 Channel Adjust Options" :resize F :transient T

   :panel P1 :layout T :padx 5 :pady 5
     :layout H; :col-gap 20; ; :row-gap 10

     :slider ADJUSTMENT  :min 0 :max 127  :value $adjustment
     :next-row T
     :choice ADJSET    :label "Set Channel Adjust"      :strings [list All Selected] :value $adjset
     :checkbox ADJENB    :label "Enable Channel Adjust" 

   :panel P2 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

        :canvas PATT  :w 400 :h 270
        [:path PATT] config -background $env(tkframebackground)

        set x 20
        set y 20
        set i 0
        for {set ii 0} {$ii < 8} {incr ii} {

           :draw text :value $x $y -text "$i"
           incr x 30

           for {set jj 0} {$jj < 16} {incr jj} {
              entry [:path PATT].bit$i -relief flat -width 1 -justify right
              [:path PATT].bit$i insert 0 "0"
              :draw window :tag PATTbit$i :value $x $y -window [:path PATT].bit$i 
              bind [:path PATT].bit$i <ButtonPress-1>  "click-opt5x-select $i"
              incr x 20
              incr i
           }

           incr x 10

           :draw text :value $x $y -text "[expr $i-1]"

           incr y 32
           set x 20
        }

   :panel P3 :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 20

     :button UPDATEOPT5x   :label "Update"   :command redisplay_opt5x
     :number ADJVAL :label "adjustment for channel" :min 1 :max 128 :w 3
     :col-gap 0
     :text   ADJVALUE :label "" :w 4 :deaf T

  :show T

  redisplay_opt5x
}

proc make-electronics {} {
  global env electronicsframe electronicsvisible

 set valuex 32c

  :frame "$electronicsframe" :label "CAEN V767 TDC Electronics" :resize F :transient T

   :panel P20 :layout T :padx 5 :pady 5
     :layout V; :row-gap 5; :col-gap 10

      :text GEOADDRESS :label "GEO Address" :w 8 :valuex $valuex
      :text INTRLEV    :label "Interrupt level" :w 8 :valuex $valuex
      :text INTRVEC    :label "Interrupt vector" :w 8 :valuex $valuex

      :choice TERM     :label "Control bus termination" :valuex $valuex :deaf T \
          :strings [list "incorrect" "OFF" "ON" "incorrect"]
      :checkbox DREADY :label "Output Buffer has data" :valuex $valuex :deaf T
      :checkbox BUSY   :label "Module Busy" :valuex $valuex :deaf T
      :checkbox GDREADY :label "A module has Data Ready" :valuex $valuex :deaf T
      :checkbox GBUSY   :label "A module is Busy" :valuex $valuex :deaf T

      :checkbox BEMPTY   :label "Output Buffer Empty" :valuex $valuex :deaf T
      :checkbox BFULL    :label "Output Buffer Full" :valuex $valuex :deaf T 
      :checkbox BPART    :label "Output Buffer Almost Full" :valuex $valuex :deaf T 
      :checkbox TDCERR   :label "TDC error" :valuex $valuex :deaf T

      :choice MCST  :label "Board MCST/CBLT status"  :valuex $valuex  :strings \
          [list "disabled" "first" "last" "intermediate"] :deaf T

      :text DRMODE     :label "Data Ready Mode"  :valuex $valuex :deaf T
      :choice AUTOLOAD  :label "Auto Load" :valuex $valuex :strings [list Disabled Enabled] :deaf T

     :layout H; :next-row T

     :button RESET    :label "Module Reset"         :command reset-module
     :button CLEAR    :label "Module clear"

     :next-row T
     :button UPDATE   :label "Electronics Update"   :command redisplay-electronics
     :button SHOWREGS :label "Show register values" :command click-registers

  :show T
 set electronicsvisible 1
}
